@@ -516,7 +516,7 @@ gst_dvdemux_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
   gst_object_unref (dvdemux);
@@ -565,7 +565,7 @@ gst_dvdemux_sink_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
   gst_object_unref (dvdemux);
@@ -1053,6 +1053,7 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       dvdemux->decoder = dv_decoder_new (0, FALSE, FALSE);
+      dv_set_error_log (dvdemux->decoder, NULL);
       dvdemux->audio_offset = 0;
       dvdemux->video_offset = 0;
       dvdemux->framecount = 0;
