@@ -41,7 +41,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_test_debug);
   (G_TYPE_INSTANCE_GET_CLASS ((obj),GST_TYPE_TEST,GstTestClass))
 #define GST_IS_TEST(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_TEST))
-#define GST_IS_TEST_CLASS(klass) \
+#define GST_IS_TEST_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_TEST))
 
 typedef struct _GstTest GstTest;
@@ -62,8 +62,6 @@ struct _GstTestClass
   gchar *param_names[2 * TESTS_COUNT];
 };
 
-static void gst_test_finalize (GstTest * test);
-
 static gboolean gst_test_start (GstBaseSink * trans);
 static gboolean gst_test_stop (GstBaseSink * trans);
 static gboolean gst_test_sink_event (GstBaseSink * basesink, GstEvent * event);
@@ -81,15 +79,14 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS_ANY);
 
 
-static const GstElementDetails details = GST_ELEMENT_DETAILS ("Test plugin",
+static GstElementDetails details = GST_ELEMENT_DETAILS ("gsttestsink",
     "Testing",
     "perform a number of tests",
     "Benjamin Otte <otte@gnome>");
 
-GST_BOILERPLATE (GstTest, gst_test, GstBaseSink, GST_TYPE_BASE_SINK);
+GST_BOILERPLATE (GstTest, gst_test, GstBaseSink, GST_TYPE_BASE_SINK)
 
-static void
-gst_test_base_init (gpointer g_class)
+     static void gst_test_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
@@ -109,8 +106,6 @@ gst_test_class_init (GstTestClass * klass)
   object_class->set_property = GST_DEBUG_FUNCPTR (gst_test_set_property);
   object_class->get_property = GST_DEBUG_FUNCPTR (gst_test_get_property);
 
-  object_class->finalize = (GObjectFinalizeFunc) gst_test_finalize;
-
   for (i = 0; i < TESTS_COUNT; i++) {
     GParamSpec *spec;
 
@@ -144,18 +139,6 @@ gst_test_init (GstTest * test, GstTestClass * g_class)
   }
 }
 
-static void
-gst_test_finalize (GstTest * test)
-{
-  guint i;
-
-  for (i = 0; i < TESTS_COUNT; i++) {
-    g_value_unset (&test->values[i]);
-  }
-
-  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) test);
-}
-
 static void
 tests_unset (GstTest * test)
 {
@@ -312,7 +295,7 @@ gst_test_get_property (GObject * object, guint prop_id, GValue * value,
 }
 
 gboolean
-gst_test_plugin_init (GstPlugin * plugin)
+gst_test_plugin_init (GstPlugin * plugin, GstPluginClass * g_class)
 {
   if (!gst_element_register (plugin, "testsink", GST_RANK_NONE, GST_TYPE_TEST))
     return FALSE;
